#!/usr/bin/perl -w

################################################################################                            
#-------------------------------------------------------------------------------                            
#  NAME:         verifyRequest.pl
#                                                                                                           
#  TYPE:         perl Script                                                                                
#                                                                                                           
#  DESCRIPTION:  This file simply displays a set of input to the user and asks
#                them to confirm that the input they gave is correct  
#
#  DEPENDENCES:  getLogFile.pm                                                                              
#                                                                                                           
#  INPUTS:       $volcano:         Name for volcano that erupted                                            
#                $erupt_year:      Year of the eruption                                                     
#                $erupt_month:     Month of the eruption                                                    
#                $erupt_day:       Day of the eruption                                                      
#                $erupt_hour:      Hour of the eruption                                                     
#                $erupt_min:       Minute of the eruption                                                   
#                $erupt_duration:  The duration of the eruption    
#                $sensor:          Sensor data being used                                                   
#                $orbit_number     Satellite data orbital number                                            
#                $product          Type of product being used                                               
#                $obs_year         Observation year                                                         
#                $obs_month        Observation month                                                        
#                $obs_day          Observation day                                                          
#                $obs_hour         Observation hour                                                         
#                $obs_min          Observation minute                                                       
#                $duCtOff          Threshold cut-off value for satellite data                               
#                                                                                                           
#  OUTPUTS:      N/A                                                                                        
#                                                                                                           
#  HISTORY:                                                                                                 
#                                                                                                           
#                                                                                                           
#-------------------------------------------------------------------------------                            
################################################################################                            


# SET FOR USAGE WITH THE CGI-BIN
use CGI;

# SET TO DISPLAY ERROR INFORMATION WHEN AN ERROR OCCURS. OTHERWISE, A
#   A STANDARD ERROR MESSAGE WILL BE USED
use CGI::Carp qw(warningsToBrowser fatalsToBrowser); 
use CGI qw(:standard);

# RETRIEVE A SET OF FUNCTIONS USED TO READ AND WRITE TO 
# LOG FILES
require "getLogFile.pm";


# GET THE SYSTEM OUTPUT DIRECTORY
my $web_puff_dir = getWebOutputDir();


################################
# MAKE SURE THE SYSTEM IS NOT
# ALREADY PROCESSING ANOTHER 
# REQUEST
################################

# CHECK TO SEE IF ANOTHER PROCESS IS WAITING TO BE RUN
my $waiting4Puff = &getWaitingLog();

# CHECK TO SEE IF ANOTHER PROCESS IS CURRENTLY BEING RUN
my $waiting2Finish = &getBegunLog();


# IF NO OTHER PROCESSES ARE CURRENTLY, OR WAITING TO, BE RUN THEN
#   ALLOW THE USER TO CONTINUE
if ($waiting4Puff == 0)
{
    if ($waiting2Finish == 0)
    {
	
	# RETRIEVE USER INPUTS FROM THE 'POST'
	my $volcano = param("volcano");                # NAME OF VOLCANO
	my $erupt_year     = param("erupt_year");      # ERUPTION YEAR
	my $erupt_month    = param("erupt_month");     # ERUPTION MONTH
	my $erupt_day      = param("erupt_day");       # ERUPTION DAY
	my $erupt_hour     = param("erupt_hour");      # ERUPTION HOUR
	my $erupt_min      = param("erupt_min");       # ERUPTION MINUTE
	my $erupt_duration = param("erupt_duration");  # DURATION OF THE ERUPTION
	my $sensor       = param("sensor");            # NAME OF SATELLITE SENSOR USED
	my $orbit_number = param("orbit_number");      # SATELLITE DATA ORBITAL NUMBER
	my $product      = param("product");           # DATA PRODUCT USED
	my $obs_year     = param("obs_year");          # OBSERVATION YEAR
	my $obs_month    = param("obs_month");         # OBSERVATION MONTH
	my $obs_day      = param("obs_day");           # OBSERVATION DAY
	my $obs_hour     = param("obs_hour");          # OBSERVATION HOUR
	my $obs_min      = param("obs_min");           # OBSERVATION MINUTE
	my $duCtOff      = param("duCtOff");           # SATELLITE DATA THRESHOLD VALUE
	my $metModel = param("metModel");              # METEROLOGICAL MODEL USED
	my $region = param("region");                  # REGION OF INTEREST
	my $hiLat  = '';                               # MAPPING LIMIT: UPPER LATITUDE
	my $lowLat = '';                               # MAPPING LIMIT: LOWER LATITUDE
	my $hiLon  = '';                               # MAPPING LIMIT: HIGHER LONGITUDE
	my $lowLon = '';                               # MAPPING LIMIT: LOWER LONGITUDE


	#############################################################                                               
        # MAKE SURE USER INPUTS ARE VALID                                                                           
        #############################################################                                               
  
        # DEFINE ERROR VARIABLES  
	my $errorFound = 0;  # ERROR FLAG 
	my $errorMsg = ' ';  # ERROR MESSAGE
	my $tmpMess = ' ';   # TEMPORARY ERROR MESSAGE

	# RETRIEVE THE MAPPING LIMITS
	if ($region ne "-----")
	{

	    # RETRIEVE THE LIMITS FOR THE SPECIFIED REGION
	    my $mapLimits = `/usr/bin/perl getRegionLimits.pl $region`;

	    # EXTRACT THE OUTPUT INTO SEPARATE LIMITS
	    my @limits = split(',',$mapLimits);
	    
	    $hiLat  = $limits[1];
	    $lowLat = $limits[0];
	    $hiLon  = $limits[3];
	    $lowLon = $limits[2];
  
	    $tmpMess=$region."__"."$mapLimits - $hiLat,$lowLat,$hiLon,$lowLon" ;

	}
	else  {

	    if 	($region == "-----")  {
		
		# IF NO REGION IS SPECIFIED, THEN RETRIEVE THE DIRECT
		# LIMITS FROM THE USER INPUTS
		$hiLat  = param("hiLat");
		$lowLat = param("lowLat");
		$hiLon  = param("hiLon");
		$lowLon = param("lowLon");		
	    }	
	    else  {
		# PRINT OUT ERROR MESSAGE, THAT THE SYSTEM HAS RECIEVED AN ERROR
	    }
	}
	

	# CHECK TO SEE IF THE LAT/LON LIMITS ARE VALID
	if (($hiLat == '' && $hiLat != 0) || ($lowLat == '' && $hiLat != 0) || ($hiLon == '' && $hiLat != 0)  || ($lowLon == '' && $hiLat != 0)) {
	    
	    $errorFound=1;
            $errorMsg = $errorMsg."The Latitude/Longitude limits entered are incomplete, ";
            $errorMsg = $errorMsg." please re-enter the limits <u>or</u> sleect a predefined region. <br>";
	}
       

	################################
        # GET THE TIME LIMITS OF THE
        # DATA ON THE MODEL SERVER
        ################################

	# DEFINE THE MINIMUM/MAXIMUM DATE-TIME
	my $minYYYY;  # MINIMUM YEAR
	my $minMM;    # MINIMUM MONTH
	my $minDD;    # MINIMUM DAY
	my $minHH;    # MINIMUM HOUR

	my $maxYYYY;  # MAXIMUM YEAR
	my $maxMM;    # MAXIMUM MONTH
	my $maxDD;    # MAXIMUM DAY
	my $maxHH;    # MAXIMUM HOUR

	# GET THE MINIMUM GFS TIME
	($minYYYY,$minMM,$minDD,$minHH)= &getGfsMinTime();

        # GET THE MAXIMUM GFS TIME
	($maxYYYY,$maxMM,$maxDD,$maxHH)= &getGfsMaxTime();

	# COMPUTE THE JULIAN DATE FOR THE RESPECTIVE BOUNDS	
	my $minJulDate = `/usr/bin/perl getJulDate.pl $minYYYY $minMM $minDD $minHH $minMN`;
	my $maxJulDate = `/usr/bin/perl getJulDate.pl $maxYYYY $maxMM $maxDD $maxHH $maxMN`;

	# COMPUTE THE JULIAN DATE FOR THE USER INPUTTED TIMES
	my $eruptJulDate = `/usr/bin/perl getJulDate.pl $erupt_year $erupt_month $erupt_day $erupt_hour $erupt_min`;
	my $obsJulDate = `/usr/bin/perl getJulDate.pl $obs_year $obs_month $obs_day $obs_hour $obs_min`;

        # ADD 6 HOURS TO THE MAX TIME, EACH FILE HAS A FORECAST TIME-RANGE
	$maxJulDate = $maxJulDate+.25;
	

	################################ 
        # MAKE SURE THE INPUTS ARE VALID
        ################################

	# CHECK TO SEE IF THE INPUTTED ERUPTION DATE IS VALID FOR THE DATA THAT
	# IS AVAILABLE 
	if (($eruptJulDate <= $minJulDate) || ($eruptJulDate >= $maxJulDate)) {
	    
	    $errorFound=1;
            $errorMsg = $errorMsg."There is no stored GFS data for the inputted <u>eruption time</u>. <br>";
	}
       

	# CHECK TO SEE IF THE INPUTTED OBSERVATION DATE IS VALID FOR THE DATA THAT
	# IS AVAILABLE 
	if (($obsJulDate <= $minJulDate) || ($obsJulDate >= $maxJulDate)) {
	    
	    $errorFound=1;
            $errorMsg = $errorMsg."There is no stored GFS data for the inputted <u>observation time</u>. <br>";
	}


	# MAKE SURE THAT A VALUE WAS ENTERED FOR THE ERUPTION DURATION 
	if ($erupt_duration == '') {
	    
	    $errorFound=1;
	    $errorMsg=$errorMsg."No eruption time was entered.  <br>";
	}
	else {
	    # MAKE SURE THE ERUPTION DURATION ENTERED ISN'T TOO LARGE (e.g less than 4 days)
	    if ($erupt_duration > 96) {
		
		$errorFound=1;
		$errorMsg=$errorMsg."The eruption duration is limited to less than 96 hours. <br>";
	    }
	}
	 
	# MAKE SURE THAT A VALUE FOR THE DU-CUTOFF HAS BEEN ENTERED
	if ( $duCtOff == '') {
	    
	    $errorFound=1;
	    $errorMsg=$errorMsg."A value for the DU-Cutoff wasn't entered. <br>";
	}	
	
	# IF ANY ERROR ARE DETECTED, THE OUTPUT AN ERROR MESSAGE
	if ($errorFound) {
	    
	    print "Content-type: text/html\n\n";

	    print "<html> \n";
	    print "<head> \n";
	    print "<title> PUFF Volcanic Height Analysis </title>  \n";
	    print "<link rel=stylesheet type=text/css href=http://satepsanone.nesdis.noaa.gov/$web_puff_dir/puffInputForm.css /> \n ";
	    print "</head>  \n";

	    print "<body>  \n";
	    print "<center>   \n";
	    print "<br></br>  \n";

	    print "<table> <tr> <td align=left>";
	    print "<b> The following errors were found: </b> <br>\n";
	    print "      $errorMsg <br> \n";
	    print "</td></tr></table>";


	    print "$tmpMess  \n";

	    print "</center>   \n";
	    print "</body>   \n";
	    print "</html>   \n";

	}

	# OTHERWISE, NO ERRORS WERE FOUND.
	else {
	    
	    # PRINT THE INPUTS TO THE USER IN THE FORM OF AN HTML PAGE
	    
	    print "Content-type: text/html\n\n";
	    
	    print "<html> \n";
	    print "<head> \n";
	    print "<title> PUFF Volcanic Height Analysis </title>  \n";
	    print "<link rel=stylesheet type=text/css href=http://satepsanone.nesdis.noaa.gov/$web_puff_dir/puffInputForm.css /> \n ";
	    print "</head>  \n";
	    
	    print "<body>  \n";
	    
	    print "<center>   \n";
	    print "<fieldset>   \n";
	    print "<legend><b>  Verify Input Parameters: </b></legend>   \n";
	    print "<table border=0>   \n";
	    print "<tr><td>   \n";
	    
	    print "<br>   \n";
	    print "<b> Eruption Parameters: </b> <br>   \n";
	    print "Volcano:   $volcano    <br>   \n"  ;
	    print "Eruption Date <br>   \n";
	    print "Year:      $erupt_year <br>   \n" ;
	    print "Month:     $erupt_month <br>   \n";
	    print "Day:       $erupt_day  <br>   \n";
	    print "Hour:      $erupt_hour  <br>   \n";
	    print "Minute    $erupt_min   <br>   \n";
	    print "<br>   \n";
	    print "DU Cutoff: $duCtOff DU <br>   \n";
	    print "<br>   \n";
	    print "<br>   \n";
	    
	    print "Eruption Duration:  $erupt_duration  <br>   \n";
	    print "<br>   \n";
	    
	    print "<b> Satellite/Observation Parameters </b> <br>   \n";
	    print "Sensor:   $sensor    <br>   \n"  ;
	    print "Product:  $product  <br>   \n";
	    print "Year:     $obs_year <br>   \n" ;
	    print "Month:    $obs_month <br>   \n";
	    print "Day:      $obs_day  <br>   \n";
	    print "Hour:     $obs_hour  <br>   \n";
	    print "Minute:   $obs_min   <br>   \n";
	    print "<br>   \n";
	    
	    print "<b> Meterological Model Parameters: </b>  <br>   \n";
	    print "Meterological Model:  $metModel  <br>   \n";
	    print "<br>   \n";
	    
	    print "Region: $region <br>   \n";
	    print "<br>   \n"; 
	    
	    print "Lat Max:  $hiLat <br>   \n";
	    print "Lat Min:  $lowLat <br>   \n";
	    print "Lon Max:  $hiLon <br>   \n";
	    print "Lon Min:  $lowLon <br>   \n";
	    print "<br>";
	    print "<br>   \n";
	    
	    print "</td></tr>   \n";
	    
	    print "<tr>   \n";
	    print " <form method=post action=submitRequest.pl>   \n";
	    
	    print "<input type=hidden name=volcano value=$volcano>   \n";
	    print "<input type=hidden name=erupt_year value=$erupt_year>    \n";
	    print "<input type=hidden name=erupt_month value=$erupt_month>   \n";
	    print "<input type=hidden name=erupt_day value=$erupt_day>   \n";
	    print "<input type=hidden name=erupt_hour value=$erupt_hour>   \n";
	    print "<input type=hidden name=erupt_min value=$erupt_min>   \n";
	    print "<input type=hidden name=erupt_duration value=$erupt_duration>   \n";
	    
	    print "<input type=hidden name=sensor value=$sensor>   \n";
	    print "<input type=hidden name=orbit_number value=$orbit_number>   \n";
	    print "<input type=hidden name=product value=$product>   \n";
	    print "<input type=hidden name=obs_year value=$obs_year>   \n";
	    print "<input type=hidden name=obs_month value=$obs_month>   \n";
	    print "<input type=hidden name=obs_day value=$obs_day>   \n";
	    print "<input type=hidden name=obs_hour value=$obs_hour>   \n";
	    print "<input type=hidden name=obs_min value=$obs_min>   \n";
	    
	    print "<input type=hidden name=duCtOff value=$duCtOff>   \n";
	    print "<input type=hidden name=metModel value=$metModel>   \n";
	    print "<input type=hidden name=hiLat value=$hiLat>   \n";
	    print "<input type=hidden name=lowLat value=$lowLat>   \n";
	    print "<input type=hidden name=hiLon value=$hiLon>   \n";
	    print "<input type=hidden name=lowLon value=$lowLon>   \n";
	    
	    print "<td align=center colspan=2> <button type=submit> Begin Processing </button> </td>   \n";
	    print "</tr>";
	    
	    print "</table>   \n";
	    print "</fieldset>   \n";
	    
	    print "</center>   \n";
	    
	    print "</body>   \n";
	    print "</html>   \n";
	}
    }
    else {
	
	# THE SYSTEM IS CURRENTLY PROCESSING ANOTHER REQUEST, TRY AGAIN LATER

	print "Content-type: text/html\n\n";
	
	print "<html>   \n";
	print "<body>   \n";
	
	print "<center>   \n";
	print "Another simulation is corrently being processed, please try again in a few minutes ...   \n";
	print "</center>   \n";
	print "</body>   \n";
	print "</html>   \n";
	
    }   
}

else {
    
    # THE SYSTEM IS CURRENTLY PROCESSING ANOTHER REQUEST, TRY AGAIN LATER

    print "Content-type: text/html\n\n";
    
    print "<html>   \n";
    print "<body>   \n";
    
    print "<center>   \n";
    print "Another simulation is corrently being processed, please try again in a few minutes ...   \n";
    print "</center>   \n";
    print "</body>   \n";
    print "</html>   \n";
    
}



